@@ -18,13 +18,16 @@ use webcore::value::{
 };
 
 use webcore::serialization::{
-    JsSerializable,
+    JsSerialize,
     SerializedValue,
     PreallocatedArena
 };
 
 use webcore::number::{self, Number, Storage, get_storage};
 use webcore::try_from::{TryInto, TryFrom};
+use webcore::instance_of::InstanceOf;
+use webcore::array::Array;
+use webcore::object::Object;
 
 impl Serialize for Undefined {
     #[inline]
@@ -155,24 +158,27 @@ impl Serialize for Value {
             Value::Null => serializer.serialize_unit_struct( "null" ),
             Value::Bool( value ) => serializer.serialize_bool( value ),
             Value::Number( ref value ) => value.serialize( serializer ),
+            Value::Symbol( _ ) => unimplemented!( "Serialization of symbols is unimplemented!" ),
             Value::String( ref value ) => serializer.serialize_str( value ),
-            Value::Array( ref value ) => {
-                let value: Vec< Value > = value.into();
-                value.serialize( serializer )
-            },
-            Value::Object( ref value ) => {
-                let value: BTreeMap< String, Value > = value.into();
-                let mut map = try!( serializer.serialize_map( Some( value.len() ) ) );
-                for (key, value) in value {
-                    try!( map.serialize_key( &key ) );
-                    try!( map.serialize_value( &value ) );
-                }
+            Value::Reference( ref reference ) => {
+                if Array::instance_of( reference ) {
+                    let array: Array = reference.try_into().unwrap();
+                    let value: Vec< Value > = array.into();
+                    value.serialize( serializer )
+                } else if Object::instance_of( reference ) {
+                    let object: Object = reference.try_into().unwrap();
+                    let value: BTreeMap< String, Value > = object.into();
+                    let mut map = try!( serializer.serialize_map( Some( value.len() ) ) );
+                    for (key, value) in value {
+                        try!( map.serialize_key( &key ) );
+                        try!( map.serialize_value( &value ) );
+                    }
 
-                map.end()
-            },
-            Value::Reference( _ ) => {
-                let map = try!( serializer.serialize_map( None ) );
-                map.end()
+                    map.end()
+                } else {
+                    let map = try!( serializer.serialize_map( None ) );
+                    map.end()
+                }
             }
         }
     }
@@ -475,7 +481,8 @@ impl< 'a > ser::Serializer for &'a mut Serializer {
     fn serialize_newtype_variant< T: ?Sized + Serialize >( self, _name: &'static str, _variant_index: u32, variant: &'static str, value: &T ) -> Result< Self::Ok, Self::Error > {
         let mut object = BTreeMap::new();
         object.insert( String::from( variant ), to_value( &value )? );
-        Ok( Value::Object( object.into() ) )
+        let object: Object = object.into();
+        Ok( Value::Reference( object.into() ) )
     }
 
     fn serialize_seq( self, length: Option< usize > ) -> Result< Self::SerializeSeq, Self::Error > {
@@ -614,8 +621,8 @@ impl ser::SerializeTupleVariant for SerializeTupleVariant {
 
     fn end( self ) -> Result< Self::Ok, Self::Error > {
         let mut object: BTreeMap< String, Value > = BTreeMap::new();
-        object.insert( self.name, Value::Array( self.elements.into() ) );
-        Ok( Value::Object( object.into() ) )
+        object.insert( self.name, self.elements.into() );
+        Ok( object.into() )
     }
 }
 
@@ -653,7 +660,7 @@ impl ser::SerializeMap for SerializeMap {
     }
 
     fn end( self ) -> Result< Self::Ok, Self::Error > {
-        Ok( Value::Object( self.map.into() ) )
+        Ok( self.map.into() )
     }
 }
 
@@ -681,9 +688,9 @@ impl ser::SerializeStructVariant for SerializeStructVariant {
     }
 
     fn end( self ) -> Result< Self::Ok, Self::Error > {
-        let mut object = BTreeMap::new();
-        object.insert( self.name, Value::Object( self.map.into() ) );
-        Ok( Value::Object( object.into() ) )
+        let mut object: BTreeMap< String, Value > = BTreeMap::new();
+        object.insert( self.name, self.map.into() );
+        Ok( object.into() )
     }
 }
 
@@ -720,9 +727,8 @@ impl Value {
                     number::Storage::F64( value ) => de::Unexpected::Float( value )
                 }
             },
+            Value::Symbol( _ ) => de::Unexpected::Other( "Symbol" ),
             Value::String( ref value ) => de::Unexpected::Str( value ),
-            Value::Array( _ ) => de::Unexpected::Seq,
-            Value::Object( _ ) => de::Unexpected::Map,
             Value::Reference( _ ) => de::Unexpected::Other( "reference to a JavaScript value" )
         }
     }
@@ -738,33 +744,36 @@ impl< 'de > de::Deserializer< 'de > for Value {
             Value::Null => visitor.visit_unit(),
             Value::Bool( value ) => visitor.visit_bool( value ),
             Value::Number( value ) => de::Deserializer::deserialize_any( value, visitor ),
+            Value::Symbol( _ ) => unimplemented!( "Deserialization of symbols is unimplemented!" ),
             Value::String( value ) => visitor.visit_string( value ),
-            Value::Array( value ) => {
-                let value: Vec< _ > = value.into();
-                let length = value.len();
-                let mut deserializer = SeqDeserializer::new( value );
-                let seq = visitor.visit_seq( &mut deserializer )?;
-                let remaining = deserializer.iter.len();
-                if remaining == 0 {
-                    Ok( seq )
+            Value::Reference( reference ) => {
+                if Array::instance_of( &reference ) {
+                    let value: Array = reference.try_into().unwrap();
+                    let value: Vec< _ > = value.into();
+                    let length = value.len();
+                    let mut deserializer = SeqDeserializer::new( value );
+                    let seq = visitor.visit_seq( &mut deserializer )?;
+                    let remaining = deserializer.iter.len();
+                    if remaining == 0 {
+                        Ok( seq )
+                    } else {
+                        Err( de::Error::invalid_length( length, &"fewer elements in the array" ) )
+                    }
+                } else if Object::instance_of( &reference ) {
+                    let value: Object = reference.try_into().unwrap();
+                    let value: BTreeMap< _, _ > = value.into();
+                    let length = value.len();
+                    let mut deserializer = MapDeserializer::new( value );
+                    let map = visitor.visit_map( &mut deserializer )?;
+                    let remaining = deserializer.iter.len();
+                    if remaining == 0 {
+                        Ok( map )
+                    } else {
+                        Err( de::Error::invalid_length( length, &"fewer elements in the object" ) )
+                    }
                 } else {
-                    Err( de::Error::invalid_length( length, &"fewer elements in the array" ) )
+                    unimplemented!(); // TODO: ?
                 }
-            },
-            Value::Object( value ) => {
-                let value: BTreeMap< _, _ > = value.into();
-                let length = value.len();
-                let mut deserializer = MapDeserializer::new( value );
-                let map = visitor.visit_map( &mut deserializer )?;
-                let remaining = deserializer.iter.len();
-                if remaining == 0 {
-                    Ok( map )
-                } else {
-                    Err( de::Error::invalid_length( length, &"fewer elements in the object" ) )
-                }
-            },
-            Value::Reference( _ ) => {
-                unimplemented!(); // TODO: ?
             }
         }
     }
@@ -781,7 +790,12 @@ impl< 'de > de::Deserializer< 'de > for Value {
     #[inline]
     fn deserialize_enum< V: Visitor< 'de > >( self, _name: &str, _variants: &'static [&'static str], visitor: V ) -> Result< V::Value, Self::Error > {
         let (variant, value) = match self {
-            Value::Object( value ) => {
+            Value::Reference( reference ) => {
+                let value: Object = match reference.try_into() {
+                    Ok( object ) => object,
+                    Err( _ ) => return Err( de::Error::invalid_value( de::Unexpected::Map, &"map with a single key" ) )
+                };
+
                 let value: BTreeMap< _, _ > = value.into();
                 let mut iter = value.into_iter();
                 let (variant, value) = match iter.next() {
@@ -861,8 +875,12 @@ impl< 'de > de::VariantAccess< 'de > for VariantDeserializer {
 
     fn tuple_variant< V: Visitor< 'de > >( self, _length: usize, visitor: V ) -> Result< V::Value, Self::Error > {
         match self.value {
-            Some( Value::Array( value ) ) => {
-                de::Deserializer::deserialize_any( SeqDeserializer::new( value.into() ), visitor )
+            Some( Value::Reference( reference ) ) => {
+                let array: Array = match reference.try_into() {
+                    Ok( array ) => array,
+                    Err( _ ) => return Err( de::Error::invalid_type( de::Unexpected::UnitVariant, &"tuple variant" ) )
+                };
+                de::Deserializer::deserialize_any( SeqDeserializer::new( array.into() ), visitor )
             },
             Some( other ) => Err( de::Error::invalid_type( other.unexpected(), &"tuple variant" ) ),
             None => Err( de::Error::invalid_type( de::Unexpected::UnitVariant, &"tuple variant" ) )
@@ -871,8 +889,12 @@ impl< 'de > de::VariantAccess< 'de > for VariantDeserializer {
 
     fn struct_variant< V: Visitor< 'de > >( self, _fields: &'static [&'static str], visitor: V ) -> Result< V::Value, Self::Error > {
         match self.value {
-            Some( Value::Object( value ) ) => {
-                de::Deserializer::deserialize_any( MapDeserializer::new( value.into() ), visitor )
+            Some( Value::Reference( reference ) ) => {
+                let object: Object = match reference.try_into() {
+                    Ok( object ) => object,
+                    Err( _ ) => return Err( de::Error::invalid_type( de::Unexpected::UnitVariant, &"struct variant" ) )
+                };
+                de::Deserializer::deserialize_any( MapDeserializer::new( object.into() ), visitor )
             },
             Some( other ) => Err( de::Error::invalid_type( other.unexpected(), &"struct variant" ) ),
             _ => Err( de::Error::invalid_type( de::Unexpected::UnitVariant, &"struct variant" ) )
@@ -999,20 +1021,20 @@ macro_rules! __js_serializable_serde_boilerplate {
     (($($impl_arg:tt)*) ($($kind_arg:tt)*) ($($bounds:tt)*)) => {
         __js_serializable_boilerplate!( ($($impl_arg)*) ($($kind_arg)*) ($($bounds)*) );
 
-        impl< $($impl_arg),* > $crate::private::JsSerializable for $($kind_arg)* where $($bounds)* {
+        impl< $($impl_arg),* > $crate::private::JsSerialize for $($kind_arg)* where $($bounds)* {
             #[inline]
-            fn into_js< 'x >( &'x self, arena: &'x $crate::private::PreallocatedArena ) -> $crate::private::SerializedValue< 'x > {
+            fn _into_js< 'x >( &'x self, arena: &'x $crate::private::PreallocatedArena ) -> $crate::private::SerializedValue< 'x > {
                 let value = $crate::private::to_value( self ).unwrap();
                 let value = arena.save( value );
-                $crate::private::JsSerializable::into_js( value, arena )
+                $crate::private::JsSerialize::_into_js( value, arena )
             }
 
             #[inline]
-            fn memory_required( &self ) -> usize {
+            fn _memory_required( &self ) -> usize {
                 // TODO: This is very inefficient. The actual conversion into
                 // the Value should be only done once.
                 let value = $crate::private::to_value( self ).unwrap();
-                $crate::private::JsSerializable::memory_required( &value )
+                $crate::private::JsSerialize::_memory_required( &value )
             }
         }
 
@@ -1189,18 +1211,18 @@ impl< T: fmt::Debug > fmt::Debug for Serde< T > {
     }
 }
 
-impl< T: Serialize > JsSerializable for Serde< T > {
+impl< T: Serialize > JsSerialize for Serde< T > {
     #[inline]
-    fn into_js< 'a >( &'a self, arena: &'a PreallocatedArena ) -> SerializedValue< 'a > {
+    fn _into_js< 'a >( &'a self, arena: &'a PreallocatedArena ) -> SerializedValue< 'a > {
         let value = to_value( &self.0 ).unwrap();
         let value = arena.save( value );
-        value.into_js( arena )
+        value._into_js( arena )
     }
 
     #[inline]
-    fn memory_required( &self ) -> usize {
+    fn _memory_required( &self ) -> usize {
         let value = to_value( &self.0 ).unwrap();
-        value.memory_required()
+        value._memory_required()
     }
 }
 
